@@ -7,6 +7,7 @@ from . import utils
 
 if typing.TYPE_CHECKING:
     from .command import MolterCommand
+    from .base import MolterInjectedClient
 
 __all__ = ("MolterContext",)
 
@@ -18,7 +19,7 @@ class MolterContext:
     This does not actually inherit from `interactions._Context`.
     """
 
-    client: interactions.Client = attrs.field()
+    client: "MolterInjectedClient" = attrs.field()
     """The bot instance."""
     message: interactions.Message = attrs.field()
     """The message this represents."""
@@ -38,6 +39,8 @@ class MolterContext:
 
     invoked_name: str = attrs.field(default=None)
     """The name/alias used to invoke the command."""
+    content_parameters: str = attrs.field(default=None)
+    """The message content without the prefix or command."""
     command: "MolterCommand" = attrs.field(default=None)
     """The command invoked."""
     args: typing.List[str] = attrs.field(factory=list)
@@ -81,13 +84,6 @@ class MolterContext:
         """Returns the guild ID where the message was sent, if applicable."""
         return self.message.guild_id
 
-    @property
-    def content_parameters(self) -> str:
-        """The message content without the prefix or command."""
-        return utils.remove_prefix(
-            self.message.content, f"{self.prefix}{self.invoked_name}"
-        ).strip()
-
     @property
     def _http(self) -> interactions.HTTPClient:
         """Returns the HTTP client the client has."""
@@ -129,6 +125,9 @@ class MolterContext:
         content: typing.Optional[str] = interactions.MISSING,  # type: ignore
         *,
         tts: typing.Optional[bool] = interactions.MISSING,  # type: ignore
+        files: typing.Optional[
+            typing.Union[interactions.File, typing.List[interactions.File]]
+        ] = interactions.MISSING,  # type: ignore
         embeds: typing.Optional[
             typing.Union["interactions.Embed", typing.List["interactions.Embed"]]
         ] = interactions.MISSING,  # type: ignore
@@ -156,6 +155,8 @@ class MolterContext:
         :param tts?: Whether the message utilizes the text-to-speech Discord \
             programme or not.
         :type tts: Optional[bool]
+        :param files?: A file or list of files to be attached to the message.
+        :type files: Optional[Union[File, List[File]]]
         :param embeds?: An embed, or list of embeds for the message.
         :type embeds: Optional[Union[Embed, List[Embed]]]
         :param allowed_mentions?: The message interactions/mention limits \
@@ -172,6 +173,7 @@ class MolterContext:
         return await channel.send(
             content,
             tts=tts,
+            files=files,
             embeds=embeds,
             allowed_mentions=allowed_mentions,
             components=components,
@@ -183,6 +185,9 @@ class MolterContext:
         content: typing.Optional[str] = interactions.MISSING,  # type: ignore
         *,
         tts: typing.Optional[bool] = interactions.MISSING,  # type: ignore
+        files: typing.Optional[
+            typing.Union[interactions.File, typing.List[interactions.File]]
+        ] = interactions.MISSING,  # type: ignore
         embeds: typing.Optional[
             typing.Union["interactions.Embed", typing.List["interactions.Embed"]]
         ] = interactions.MISSING,  # type: ignore
@@ -210,6 +215,8 @@ class MolterContext:
         :param tts?: Whether the message utilizes the text-to-speech Discord \
             programme or not.
         :type tts: Optional[bool]
+        :param files?: A file or list of files to be attached to the message.
+        :type files: Optional[Union[File, List[File]]]
         :param embeds?: An embed, or list of embeds for the message.
         :type embeds: Optional[Union[Embed, List[Embed]]]
         :param allowed_mentions?: The message interactions/mention limits \
@@ -225,6 +232,7 @@ class MolterContext:
         return await self.message.reply(
             content,
             tts=tts,
+            files=files,
             embeds=embeds,
             allowed_mentions=allowed_mentions,
             components=components,
